@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002,2004 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -148,7 +148,7 @@ extern char ** environ;
 #include <locale.h>
 #endif
 
-#if HAVE_LIBCRYPT && HAVE_CRYPT_H
+#if HAVE_CRYPT_H
 #  include <crypt.h>
 #endif
 
@@ -160,6 +160,10 @@ extern char ** environ;
 # include <sys/file.h>
 #endif
 
+#if HAVE_CRT_EXTERNS_H
+#include <crt_externs.h>  /* for Darwin _NSGetEnviron */
+#endif
+
 /* Some Unix systems don't define these.  CPP hair is dangerous, but
    this seems safe enough... */
 #ifndef R_OK
@@ -197,6 +201,13 @@ extern char ** environ;
    above means that _POSIX_SOURCE may be #defined, which will
    encourage header files to do strange things.  */
 
+/* On Apple Darwin in a shared library there's no "environ" to access
+   directly, instead the address of that variable must be obtained with
+   _NSGetEnviron().  */
+#if HAVE__NSGETENVIRON && defined (PIC)
+#define environ (*_NSGetEnviron())
+#endif
+
 
 SCM_SYMBOL (sym_read_pipe, "read pipe");
 SCM_SYMBOL (sym_write_pipe, "write pipe");
@@ -895,10 +906,16 @@ SCM_DEFINE (scm_execl, "execl", 1, 0, 1,
 #define FUNC_NAME s_scm_execl
 {
   char **execargv;
+  int save_errno, i;
   SCM_VALIDATE_STRING (1, filename);
   SCM_STRING_COERCE_0TERMINATION_X (filename);
   execargv = scm_convert_exec_args (args, SCM_ARG2, FUNC_NAME);
   execv (SCM_STRING_CHARS (filename), execargv);
+  save_errno = errno;
+  for (i = 0; execargv[i] != NULL; i++)
+    free (execargv[i]);
+  free (execargv);
+  errno = save_errno;
   SCM_SYSERROR;
   /* not reached.  */
   return SCM_BOOL_F;
@@ -916,10 +933,16 @@ SCM_DEFINE (scm_execlp, "execlp", 1, 0, 1,
 #define FUNC_NAME s_scm_execlp
 {
   char **execargv;
+  int save_errno, i;
   SCM_VALIDATE_STRING (1, filename);
   SCM_STRING_COERCE_0TERMINATION_X (filename);
   execargv = scm_convert_exec_args (args, SCM_ARG2, FUNC_NAME);
   execvp (SCM_STRING_CHARS (filename), execargv);
+  save_errno = errno;
+  for (i = 0; execargv[i] != NULL; i++)
+    free (execargv[i]);
+  free (execargv);
+  errno = save_errno;
   SCM_SYSERROR;
   /* not reached.  */
   return SCM_BOOL_F;
@@ -968,6 +991,7 @@ SCM_DEFINE (scm_execle, "execle", 2, 0, 1,
 {
   char **execargv;
   char **exec_env;
+  int save_errno, i;
 
   SCM_VALIDATE_STRING (1, filename);
   SCM_STRING_COERCE_0TERMINATION_X (filename);
@@ -975,6 +999,13 @@ SCM_DEFINE (scm_execle, "execle", 2, 0, 1,
   execargv = scm_convert_exec_args (args, SCM_ARG1, FUNC_NAME);
   exec_env = environ_list_to_c (env, SCM_ARG2, FUNC_NAME);
   execve (SCM_STRING_CHARS (filename), execargv, exec_env);
+  save_errno = errno;
+  for (i = 0; execargv[i] != NULL; i++)
+    free (execargv[i]);
+  free (execargv);
+  for (i = 0; exec_env[i] != NULL; i++)
+    free (exec_env[i]);
+  free (exec_env);
   SCM_SYSERROR;
   /* not reached.  */
   return SCM_BOOL_F;
@@ -1208,6 +1239,18 @@ SCM_DEFINE (scm_putenv, "putenv", 1, 0, 0,
   char *ptr;
 
   SCM_VALIDATE_STRING (1, str);
+
+  /* No '=' in the argument means we should remove the variable from the
+     environment.  Not all putenv()s understand this (for instance FreeBSD
+     4.8 doesn't).  To be safe, we do it explicitely using unsetenv. */
+#ifdef HAVE_UNSETENV
+  if (strchr (SCM_STRING_CHARS (str), '=') == NULL)
+    {
+      unsetenv (SCM_STRING_CHARS (str));
+      return SCM_UNSPECIFIED;
+    }
+#endif
+
   /* must make a new copy to be left in the environment, safe from gc.  */
   ptr = malloc (SCM_STRING_LENGTH (str) + 1);
   if (ptr == NULL)
@@ -1324,7 +1367,11 @@ SCM_DEFINE (scm_nice, "nice", 1, 0, 0,
 #define FUNC_NAME s_scm_nice
 {
   SCM_VALIDATE_INUM (1,incr);
-  if (nice(SCM_INUM(incr)) != 0)
+  /* nice() returns "prio-NZERO" on success or -1 on error, but -1 can arise
+     from "prio-NZERO", so an error must be detected from errno changed */
+  errno = 0;
+  nice(SCM_INUM(incr));
+  if (errno != 0)
     SCM_SYSERROR;
   return SCM_UNSPECIFIED;
 }
@@ -1344,7 +1391,7 @@ SCM_DEFINE (scm_sync, "sync", 0, 0, 0,
 #undef FUNC_NAME
 #endif /* HAVE_SYNC */
 
-#if HAVE_LIBCRYPT && HAVE_CRYPT_H
+#if HAVE_CRYPT
 SCM_DEFINE (scm_crypt, "crypt", 2, 0, 0, 
             (SCM key, SCM salt),
 	    "Encrypt @var{key} using @var{salt} as the salt value to the\n"
@@ -1362,7 +1409,7 @@ SCM_DEFINE (scm_crypt, "crypt", 2, 0, 0,
   return scm_makfrom0str (p);
 }
 #undef FUNC_NAME
-#endif /* HAVE_LIBCRYPT && HAVE_CRYPT_H */
+#endif /* HAVE_CRYPT */
 
 #if HAVE_CHROOT
 SCM_DEFINE (scm_chroot, "chroot", 1, 0, 0, 
@@ -1592,7 +1639,7 @@ SCM_DEFINE (scm_gethostname, "gethostname", 0, 0, 0,
 {
   /* 256 is for Solaris, under Linux ENAMETOOLONG is returned if not
      large enough.  */
-  int len = 256, res;
+  int len = 256, res, save_errno;
   char *p = scm_must_malloc (len, "gethostname");
   SCM name;
 
@@ -1605,7 +1652,9 @@ SCM_DEFINE (scm_gethostname, "gethostname", 0, 0, 0,
     }
   if (res == -1)
     {
+      save_errno = errno;
       scm_must_free (p);
+      errno = save_errno;
       SCM_SYSERROR;
     }
   name = scm_makfrom0str (p);
