@@ -1,4 +1,4 @@
-/* Copyright (C) 1996,1997,1998,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,1998,2000,2001,2004 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -705,9 +705,7 @@ scm_fill_sockaddr (int fam, SCM address, SCM *args, int which_arg,
 	soka = (struct sockaddr_in *) malloc (sizeof (struct sockaddr_in));
 	if (!soka)
 	  scm_memory_error (proc);
-	/* 4.4BSD-style interface includes sin_len member and defines SIN_LEN,
-	   4.3BSD  does not.  */
-#ifdef SIN_LEN
+#if HAVE_STRUCT_SOCKADDR_SIN_LEN
 	soka->sin_len = sizeof (struct sockaddr_in);
 #endif
 	soka->sin_family = AF_INET;
@@ -743,7 +741,7 @@ scm_fill_sockaddr (int fam, SCM address, SCM *args, int which_arg,
 	soka = (struct sockaddr_in6 *) malloc (sizeof (struct sockaddr_in6));
 	if (!soka)
 	  scm_memory_error (proc);
-#ifdef SIN_LEN6
+#if HAVE_STRUCT_SOCKADDR_IN6_SIN6_LEN
 	soka->sin6_len = sizeof (struct sockaddr_in6);
 #endif
 	soka->sin6_family = AF_INET6;
@@ -1305,16 +1303,27 @@ scm_init_socket ()
   scm_c_define ("INADDR_LOOPBACK", scm_ulong2num (INADDR_LOOPBACK));
 #endif
 
-  /* socket types.  */
+  /* socket types.
+
+     SOCK_PACKET is deliberately omitted, the GNU/Linux socket(2) and
+     packet(7) man pages advise that it's obsolete and strongly
+     deprecated.  */
+
 #ifdef SOCK_STREAM
   scm_c_define ("SOCK_STREAM", SCM_MAKINUM (SOCK_STREAM));
 #endif
 #ifdef SOCK_DGRAM
   scm_c_define ("SOCK_DGRAM", SCM_MAKINUM (SOCK_DGRAM));
 #endif
+#ifdef SOCK_SEQPACKET
+  scm_c_define ("SOCK_SEQPACKET", SCM_MAKINUM (SOCK_SEQPACKET));
+#endif
 #ifdef SOCK_RAW
   scm_c_define ("SOCK_RAW", SCM_MAKINUM (SOCK_RAW));
 #endif
+#ifdef SOCK_RDM
+  scm_c_define ("SOCK_RDM", SCM_MAKINUM (SOCK_RDM));
+#endif
 
   /* setsockopt level.  */
 #ifdef SOL_SOCKET
